﻿using Avalonia;
using Avalonia.Controls;
using Avalonia.Interactivity;
using Avalonia.Markup.Xaml;
using Avalonia.ReactiveUI;
using Crosslight.API.IO.FileSystem;
using Crosslight.API.IO.FileSystem.Abstractions;
using Crosslight.API.Nodes.Implementations;
using Crosslight.API.Transformers;
using Crosslight.Transformer.CIL.Transformers;
using Crosslight.Transformer.Viewer.ViewModels.Graph;
using Crosslight.Transformer.Viewer.ViewModels.Viewports;
using Crosslight.Transformer.Viewer.ViewModels.Windows;
using Crosslight.Transformer.Viewer.Views.Viewports;
using ReactiveUI;
using System.Reactive.Disposables;

namespace Crosslight.Transformer.Viewer
{
    public class MainWindow : ReactiveWindow<MainWindowViewModel>
    {
        private ViewerViewport ViewerViewport => this.FindControl<ViewerViewport>("viewerViewport");
        public MainWindow()
        {
            this.WhenActivated(disposables =>
            {
                this.Bind(this.ViewModel, x => x.ViewportViewModel, x => x.ViewerViewport.ViewModel)
                    .DisposeWith(disposables);
            });
            InitializeComponent();
        }

        private void InitializeComponent()
        {
            AvaloniaXamlLoader.Load(this);
        }

        private async void OnButtonClick(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog
            {
                Title = "Choose input files",
                AllowMultiple = true
            };
            var outPathStrings = await openFileDialog.ShowAsync(this);
            if (outPathStrings.Length == 0) return;

            IDirectory source = FileSystem.FromFiles(outPathStrings);

            CrosslightContext context = new CrosslightContext()
            {
                InputTransformer = new CILInputTransformer(),
                OutputTransformer = null,
            };

            Node ast = GetNodeFromFSItem(context.InputTransformer.Translate(source));
            if (ast == null)
            {
                return;
            }

            ViewModel.ViewportViewModel = new ViewerViewportViewModel()
            {
                GraphViewModel = new GraphViewerViewModel()
                {
                    RootNode = ast,
                }
            };
        }

        private Node GetNodeFromFSItem(IFileSystemItem item)
        {
            if (item is IPhysicalFile) return null;
            if (item is IStringFile) return null;
            if (item is IFile file) return file.Content as Node;
            if (item is IDirectory directory)
            {
                foreach (var dirFile in directory.Items)
                {
                    var res = GetNodeFromFSItem(dirFile);
                    if (res != null) return res;
                }
                return null;
            }
            return null;
        }
    }
}
